/**@file
 *
 * @defgroup   iBeacon handler
 * @{
 * @ingroup 
 * @brief    Module for handling iBeacon operationality.
 *
 * @details  The module handles the storing and forwarding of the
 * components needed for an iBeacon frame.
 *
 */

#include <stdlib.h>

#include "ble_advdata.h"
#include "sdk_config.h"
#include "app_error.h"

#include "beacon_flash.h"
#include "beacon_ibeacon.h"
#include "beacon_app_typedefs.h"

#define NRF_LOG_MODULE_NAME beacon_ibeacon
#include "nrf_log.h"
NRF_LOG_MODULE_REGISTER();


/** Private definitions */

#define BEACON_IBEACON_FRAME_LENGTH                0x17                             /**< Total length of information advertised by the Beacon */
#define BEACON_IBEACON_ADV_DATA_LENGTH             0x15                             /**< Length of manufacturer specific data in the advertisement */
#define BEACON_IBEACON_DEVICE_TYPE                 0x02                             /**< 0x02 refers to iBeacon */
#define BEACON_IBEACON_DEFAULT_MEASURED_RSSI       0xd8                             /**< Measured RSSI at 1 meter distance in dBm */
#define BEACON_IBEACON_DEFAULT_COMPANY_IDENTIFIER  0x0059                           /**< Company identifier for Nordic Semiconductor */
#define BEACON_IBEACON_DEFAULT_MAJOR_0_VALUE         0x00                           /**< Major value used to identify iBeacons */
#define BEACON_IBEACON_DEFAULT_MAJOR_1_VALUE         0x00                           /**< Major value used to identify iBeacons */
#define BEACON_IBEACON_DEFAULT_MINOR_0_VALUE         0x00                           /**< Minor value used to identify iBeacons */
#define BEACON_IBEACON_DEFAULT_MINOR_1_VALUE         0x00                          /**< Minor value used to identify iBeacons */
#define BEACON_IBEACON_DEFAULT_UUID                { 0x01, 0x02, 0x03, 0x04, \
                                                0x05, 0x06, 0x07, 0x08, \
                                                0x09, 0x0a, 0x0b, 0x0c, \
                                                0x0d, 0x0e, 0x0f, 0x10 }                /**< Proprietary UUID for iBeacon */

typedef struct {
    uint8_t id;     /**< Packet ID; Equal to 2 for an iBeacon */
    uint8_t len;    /**< Length of the remaining data in the payload */
    uint8_t uid[16];      /**< Beacon UID */
    uint8_t major_0;       /**< Beacon major ID */        
    uint8_t major_1;
    uint8_t minor_0;       /**< Beacon minor ID */
    uint8_t minor_1;
    uint8_t tx;        /**< Tx power received at 1 meter in dBm */
}__attribute__((packed)) beacon_ibeacon_payload_t;

/** Private variable declarations */

static beacon_ibeacon_params_t m_ibeacon_current_params = {
    .company_id = BEACON_IBEACON_DEFAULT_COMPANY_IDENTIFIER,
    .uid = BEACON_IBEACON_DEFAULT_UUID,

    .major_0 = BEACON_IBEACON_DEFAULT_MAJOR_0_VALUE,
    .major_1 = BEACON_IBEACON_DEFAULT_MAJOR_1_VALUE,
    .minor_0 = BEACON_IBEACON_DEFAULT_MINOR_0_VALUE,
    .minor_1 = BEACON_IBEACON_DEFAULT_MINOR_1_VALUE,
};

/* The advertisement payload */
static beacon_ibeacon_payload_t m_beacon_ibeacon_payload = {

    .id = BEACON_IBEACON_DEVICE_TYPE,
    .len = BEACON_IBEACON_ADV_DATA_LENGTH,
    .uid = BEACON_IBEACON_DEFAULT_UUID,
    .major_0 =BEACON_IBEACON_DEFAULT_MAJOR_0_VALUE,    //if don't comment out this five bytes, will raise NRF_ERROR_DATA_SIZE   
	.major_1 =BEACON_IBEACON_DEFAULT_MAJOR_1_VALUE,
    .minor_0 = BEACON_IBEACON_DEFAULT_MINOR_0_VALUE,
	.minor_1 = BEACON_IBEACON_DEFAULT_MINOR_1_VALUE,
    .tx = BEACON_IBEACON_DEFAULT_MEASURED_RSSI,
};

/* Manufacturer data struct. This is passed on to the advertisement module. */
static ble_advdata_manuf_data_t m_beacon_ibeacon_manuf_data = {
        .company_identifier = BEACON_IBEACON_DEFAULT_COMPANY_IDENTIFIER,
        .data.p_data = (uint8_t *) &m_beacon_ibeacon_payload,
        .data.size   =  sizeof(beacon_ibeacon_payload_t),
};

/** Private function declarations */

/** @brief set current advertisement payload
 */
static void m_beacon_ibeacon_set_payload(void);

/** Private function implementations */

static void m_beacon_ibeacon_set_payload(void)
{
    m_beacon_ibeacon_manuf_data.company_identifier = m_ibeacon_current_params.company_id;
    memcpy(m_beacon_ibeacon_payload.uid, m_ibeacon_current_params.uid, sizeof(m_beacon_ibeacon_payload.uid));
    m_beacon_ibeacon_payload.major_0 = m_ibeacon_current_params.major_0;
    m_beacon_ibeacon_payload.major_1 = m_ibeacon_current_params.major_1;
    m_beacon_ibeacon_payload.minor_0 = m_ibeacon_current_params.minor_0;
    m_beacon_ibeacon_payload.minor_1 = m_ibeacon_current_params.minor_1;
    m_beacon_ibeacon_payload.tx = m_ibeacon_current_params.ranging_data;
}

/** Public function implementations  */

ret_code_t beacon_ibeacon_init(void)
{
    /* See if there is a config saved, and configure the module accoridingly if so */
    if(beacon_flash_access_ibeacon_params(&m_ibeacon_current_params, BEACON_FLASH_ACCESS_READ) == NRF_SUCCESS) {
        beacon_ibeacon_configure(&m_ibeacon_current_params);
    }

    return NRF_SUCCESS;
}

ret_code_t beacon_ibeacon_configure(beacon_ibeacon_params_t* params)
{
    if(params == NULL) {
        return NRF_ERROR_INVALID_ADDR;
    }
    if(memcmp(params, &m_ibeacon_current_params, sizeof(beacon_ibeacon_params_t)) == 0) {
        /* Current config matches, return success  */
        m_beacon_ibeacon_set_payload();
        return NRF_SUCCESS;
    }
    memcpy(&m_ibeacon_current_params, params, sizeof(beacon_ibeacon_params_t));
    m_beacon_ibeacon_set_payload();
    beacon_flash_access_ibeacon_params(&m_ibeacon_current_params, BEACON_FLASH_ACCESS_WRITE);

    return NRF_SUCCESS;
}

void beacon_ibeacon_get_advertisement_data(ble_advdata_manuf_data_t* p_manuf_data)
{   
    NRF_LOG_INFO("sizeof_my_maunf: %d", sizeof(m_beacon_ibeacon_manuf_data));
    memcpy(p_manuf_data, &m_beacon_ibeacon_manuf_data, sizeof(ble_advdata_manuf_data_t));

}
